from ..pygtkhelpers.delegates import SlaveView
from collections import namedtuple
import os

 # The sys.path is modified, see the __main__.py file
from .. import settings

if settings.avahi_available:
    from ..zeroconf.avahilib import AvahiServiceDiscovery as ServiceBrowser
else:
    from ..zeroconf.bonjourlib import BonjourServiceBrowser as ServiceBrowser

import gtk
import statusbar

class UserList(SlaveView):
    
    TARGETS = [
        ('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),
        ('text/plain', 0, 1),
        ('TEXT', 0, 2),
        ('STRING', 0, 3),
        ]
    
    def create_ui(self):
        model = gtk.ListStore(object)
        treeview = gtk.TreeView(model)
        treeview.set_name("User List")

        iconrend = gtk.CellRendererPixbuf()
        inforend = gtk.CellRendererText()

        iconcol = gtk.TreeViewColumn('Icon', iconrend)
        infocol = gtk.TreeViewColumn('Info', inforend)
        
        iconcol.set_cell_data_func(iconrend, self._icon_data)
        infocol.set_cell_data_func(inforend, self._info_data)
        
        treeview.append_column(iconcol)
        treeview.append_column(infocol)
        treeview.set_headers_visible(False)
        
        treeview.enable_model_drag_dest(self.TARGETS,
                                        gtk.gdk.ACTION_DEFAULT)

        self.store = model
        self.treeview = treeview
        self.widget.add(treeview)

    def get_user_at_iter(self, iter):

        return self.store[iter][0]

    def _icon_data(self, column, cell, model, iter):

        row = model[iter]
        user = row[0]
        cell.set_property("pixbuf",gtk.gdk.pixbuf_new_from_file(user.icon))
    
    def _info_data(self, column, cell, model, iter):

        row = model[iter]
        user = row[0]
        
        template = "<big><b>{user}</b></big>\n<small><i>{address}:{port}</i></small>"
        label = template.format(user=user.name,
                                address=user.address,
                                port=user.port)
        cell.set_property("markup",label)

    def get_selected_user(self):

        tree_sel = self.treeview.get_selection()
        model, iter = tree_sel.get_selected()
        if iter==None:
            return None
        else:
            return self.get_user_at_iter(iter)

    def add_user(self, user):

        self.store.append([user])
        statusbar.log("%s discovered"%user.name)
    
    def remove_user(self, name):
        '''
        remove a user by name
        '''
        for i,entry in enumerate(self.store):
            user = entry[0]
            if str(user.name) == str(name):
                self.store.remove(self.store.get_iter(i))
                statusbar.log("%s removed"%user.name)
    
    def on_treeview__drag_data_received(self,
                                        drag_context,
                                        x, y,
                                        selection_data,
                                        info, time, data):
        print "context", drag_context
        print "x y",x,y
        print "selection data",selection_data
        print "info",info
        print "time", time
        print "data", data

class UserDiscovery(ServiceBrowser):

    def __init__(self,userlist):

        ServiceBrowser.__init__(self,"_filesnake._tcp")
        self.userlist = userlist

    def on_service_found(self, service_info):

        dir = os.path.dirname(__file__)
        user = User(
            name=service_info.name,
            icon=os.path.join(dir,"glade","images","gnome-laptop.png"),
            address=service_info.address,
            port=service_info.port)
        self.userlist.add_user(user)

    def on_error(self, err):

        statusbar.log("error: %s"%err)

    def on_service_removed(self, service):

        self.userlist.remove_user(service.name)

    def on_discovery_finished(self):

        statusbar.log("Done")

User = namedtuple("User", "name icon address port")
