'''
Twisted trial testing helpers

'''

from twisted.trial import unittest
from twisted.internet import reactor
from twisted.spread import pb
from abc import ABCMeta,abstractproperty
from twisted.test import proto_helpers
# For exporting
StringTransport = proto_helpers.StringTransport

class ServerTestCase(unittest.TestCase):
    __metaclass__ = ABCMeta
    
    server_factory = abstractproperty()
    port = abstractproperty()
    
    def setUp(self):
        self._serv_handler = reactor.listenTCP(self.port,
                                               self.server_factory)
    def tearDown(self):
        self._serv_handler.stopListening()

class ProtocolTestCase(unittest.TestCase):
    '''
    This is useful for testing protocols, defines the function setUpFac,
    by default it calls the function
    
    Ideas:
    - An helper method to define something like a "mock factory"?
    - Splitting the self.proto.makeConnection in another function?
      look at FileClientTestCase in file fileserver_test.py
    '''
    def setUp(self):
        if hasattr(self, "factory"):
            self.setUpFac(self.factory)

    def setUpFac(self, factory):
        self.proto = factory.buildProtocol(None)
        self.tr = proto_helpers.StringTransport()
        self.proto.makeConnection(self.tr)

        
class LineReceiverTestCase(ProtocolTestCase):
    
    def assertLineReceiveResponse(self, received, response, message = ''):
        self.proto.lineReceived(received)
        self.assertEquals(self.tr.value(), response, message)
        self.tr.clear()

#XXX calling it mixin?
class ClientTestCase(unittest.TestCase):

    def runServer(self, factory, port):
        self._listening2 = reactor.listenTCP(port, factory)
        self.addCleanup(self._listening2.stopListening)
        return self._listening2

    def runPBServer(self, pbroot, port):
        #XXX Duplicated code
        # Running the server
        self.serv_factory = pb.PBServerFactory(pbroot)
        self._listening = reactor.listenTCP(port, self.serv_factory)
        self.addCleanup(self._listening.stopListening)
        return self._listening

class PBRootTestCase(unittest.TestCase):
    '''
    This is a useful test case to handle the spawning of PB services.
    
    there should be some utilities to stop trial to tell us that he is
    dirty
    '''

    def setUp(self):
        '''
        Starts the Server specified
        '''
        self.serv_factory = pb.PBServerFactory(self.pbroot)
        self._listening = reactor.listenTCP(self.port, self.serv_factory)
        self.client_factory = pb.PBClientFactory()
        self._connect = reactor.connectTCP("localhost", self.port, 
                                           self.client_factory)
        d = self.client_factory.getRootObject()
        
        def assign(root):
            self.root = root
        d.addCallback(assign)
        return d
    
    def tearDown(self):
        '''
        Kill the server, note: stopListening() returns a callback and
        you have to return it!
        '''
        self._connect.disconnect()
        return self._listening.stopListening()
    
    # Should be moved in a superclass
    def assertDeferEquals(self, deferred, value, message=""):
        def assertion(res):
            self.assertEquals(res, value, message)
        deferred.addCallback(assertion)
        return deferred


# File That's nice
import os
def make_file(path="/tmp/text.txt", size=10000):
    if os.path.exists(path) and os.path.getsize(path) == size:
       return path,size
    
    fd = open(path, "wb")
    fd.write("0"*size)
    fd.close()
    return path, size

def make_example_folder():
    os.mkdir("/tmp/testfolder")
    
    
