from twisted.protocols.basic import LineReceiver,FileSender
from twisted.internet import defer
import logging
import re
from twisted.internet import reactor

logger = logging.getLogger("networking")
cmdregex = re.compile("(?P<command>CHECK|RESOURCE):(?P<secret>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})(?P<args>.*)")
dataregex=re.compile("(?P<secret>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})")

class FileServerProtocol(LineReceiver):
    '''
    Protocol used to send files
    
    it dispatch events to various sources like a monitor for file sendings
    '''
    def connectionMade(self):
        logger.debug("FileServerProtocol file connection made")
    
    def lineReceived(self,line):
        match = cmdregex.match(line) 
        if match:
            command = match.group("command")
            secret = match.group("secret")
            args = match.group("args")
            
            if command=="CHECK":
                self.check(secret)
            if command=="RESOURCE":
                # Optional handling of args
                self.receiving(secret,args)
        else:
            self.error()

    def error(self,msg=""):
        self.sendLine("ERROR")
        

    def receiving(self, secret, fname):
        if self.authorized(secret):
            # Preparing the connection for receiving the next file
            self.file_handler = self.factory.get_file_handler(secret)
            self.monitor = self.factory.get_monitor(secret)
            self.file_handler.dest = fname
            self.sendLine("RESOURCE:OK")
            self.setRawMode()
        else:
            self.sendLine("CHECK:NO")
        
    def authorized(self, secret):
        return self.factory.check(secret)
    
    def check(self, secret):
        if self.authorized(secret):
            self.sendLine("CHECK:OK")
        else:
            self.sendLine("CHECK:NO")
            self.transport.loseConnection()
        
    def rawDataReceived(self, data):
        # Write the data somewhere 
        self.monitor.producing(data)
        self.file_handler.save_data(data)
        # Should be processing for incoming events

    def pause(self):
        '''
        TODO: implement this, maybe sending a line to the client
        telling to stop producing... something like that
        '''


class FileClientProtocol(LineReceiver):
    '''
    The code can be refactored in a state machine, or something like
    that.
    '''

    def lineReceived(self,line):
        if line == "CHECK:OK":
            self._auth_d.callback(True)
        elif line == "CHECK:NO":
            self._auth_d.callback(False)
        elif line == "RESOURCE:OK":
            return
        else:
            #print line
            pass

        #del self._auth_d

    def authorize(self, secret):
        
        self.secret = secret
        self.sendLine("CHECK:"+secret)
        
        #self._auth_d.addErrback(log.err)
        return self._auth_d

    def connectionMade(self):        
        self._auth_d = defer.Deferred()

    
    def send_resource(self, dest, stream, monitor=None):
        
        if monitor:
            fs = MonitoredFileSender(monitor)
        else:
            fs = FileSender()
        
        def transform(data):
            if monitor:
                monitor.producing(data)
            return data
        
        # Announcing the filename
        self.sendLine("RESOURCE:"+self.secret+dest)
        #self.setRawMode()
        defer = fs.beginFileTransfer(stream, self.transport, transform=transform)
        # Disconnecting when finished
        defer.addCallback(lambda lastbyte: self.transport.loseConnection())
        return defer

class MonitoredFileSender(FileSender):
    '''
    This file sender takes a monitor that has to respect the interface
    IMonitor
    '''
    def __init__(self, monitor):
        self.monitor = monitor
        self.position = 0 # Position in bytes
        
    def resumeProducing(self):
        FileSender.resumeProducing(self)
        
    def pauseProducing(self):
        self.monitor.pause()
    
    def stopProducing(self):
        self.monitor.stop()
        #FileSender.stopProducing(self)



