'''
Tests the file sending client and the folder sending client.

file sending client is based on the low_level protocols

folder sending one is based on the perspective broker to make dirs and
on the reuse of low_level protocol (with little modification)

the authorization is done externally, or I have to make it internal?
NO, we can send different files with the same authorization session.
'''

'''
Drafting an API
The client is used by the GUI

MainClient:
autheticate(size, info) -> secret
send_file(session_id, file, monitor = None)
send_folder(session_id, folder, monitor = None)

'''
from filesnake.helpers.testing import ClientTestCase, make_file
from filesnake.networking.client import SendingClient
from filesnake.networking.server import FileReceiverRoot, SessionManager
from twisted.internet import defer
from twisted.trial import unittest
from mock import Mock
from filesnake.networking.server import FileServerFactory
from filesnake.helpers.path import get_size
import os
TESTFILE = "/tmp/file.txt"
make_file(TESTFILE, 100)

class ClientTest(ClientTestCase):
    
    def setUp(self):
        # Passing a monitor? Monitor factory?
        session_manager = Mock(SessionManager())
        session_manager.get.return_value = dict(file_handler = Mock(), 
                                                monitor = Mock())
        
        ui = Mock()
        d2 = self.runServer(FileServerFactory(ui, session_manager),10001)

        d = self.runPBServer(FileReceiverRoot(session_manager, ui, 10001), 10000)
        self.client = SendingClient(
            "galois",
            "localhost",
            10000,
            ) 
        return d,d2
        
    def test_authorize(self):
        return self.client.request_session(TESTFILE,100)
    
    def test_send_file(self):
        d = self.client.request_session(TESTFILE, 100)
        d.addCallback(
            lambda secret: self.client.send_file(secret, TESTFILE))
        return d
    
    def test_send_folder(self):
        return self.client.send_folder("secret","aou")
    
    def test_send_file_monitored(self):
        mon = Mock()
        d = self.client.send_file("secret", TESTFILE, monitor=mon)
        return d
    test_send_file_monitored.skip = "Monitor filters the stuff so it can't be ok"
    def test_folder_monitored(self):
        mon = Mock()
        d = self.client.send_folder("secret", "aou", monitor=mon)
        return d

    def tearDown(self):
        if os.path.exists("/tmp/aou"):
            os.rmdir("/tmp/aou")
        
        try:
            self.client.disconnect()
        except:
            pass
