from filesnake.helpers.testing import LineReceiverTestCase,StringTransport
from filesnake.networking.server import FileServerFactory, SessionManager
from filesnake.networking.fileprotocol import MonitoredFileSender
from mock import Mock
from twisted.trial import unittest
from uuid import uuid4

uuuid_regex="[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}"

class FileProtocolTestCase(LineReceiverTestCase):
    
    @property
    def factory(self):
        if hasattr(self, "_factory"):
            return self._factory
        controller = Mock(spec=["get_file_handler", "get_monitor", "authorize"])
        session_manager = SessionManager()
        #session_manager.get.return_value = dict(file_handler = Mock(), monitor = Mock())
        self._factory = FileServerFactory(controller,session_manager)
        return self._factory
    
    def _set_auth(self, secret):
        '''
        Set a session id for the mock session_manager
        '''
        self.factory.session_manager.register(secret, dict(monitor=Mock(),
                                                           file_handler=Mock(),
                                                           authorized=True))
    def _get_uuids(self, n=1):
        ''' returns a number of uuid in string format'''
        return [str(uuid4()) for i in range(n)] if n>1 else str(uuid4())
    def test_check_authorization(self):
        key, key_false = self._get_uuids(2)
        # Auth with the first key
        self._set_auth(key)
        self.assertLineReceiveResponse("CHECK:"+key_false, "CHECK:NO\r\n")
        self.assertLineReceiveResponse("CHECK:"+key, "CHECK:OK\r\n")
    
    def test_resource(self):
        key, key_false = self._get_uuids(2)
        
        self._set_auth(key)
        self.assertLineReceiveResponse("RESOURCE:"+key+"test/stuff",
                                       "RESOURCE:OK\r\n")
        self.assertLineReceiveResponse("RESOURCE:"+key_false+"test/stuff",
                                       "CHECK:NO\r\n")
    
    def test_error(self):
        self.assertLineReceiveResponse("aksdnfb", "ERROR\r\n")

# Difficult to test
class MFileSenderTestCase(unittest.TestCase):
    
    def test_sending(self):

        monitor = Mock()
        fs = MonitoredFileSender(monitor)
        tr = StringTransport()
        d = fs.beginFileTransfer(open(__file__,"rb"), tr)
        d.addCallback(lambda _: fs.resumeProducing())
        d.addCallback(lambda _: self.assertEquals(tr.value(),1))
        return d
    test_sending.skip ="Testing producer/consumer isn't so easy"        

from filesnake.networking.server import Server
class TestServer(unittest.TestCase):
    
    def test_init(self):
        uimanager = Mock(spec=["get_file_handler", "get_monitor", "authorize"])
        s = Server(uimanager,10000)
        s.stop_listening()
        
if __name__ == '__main__':
    unittest.main()
