from generic import ServiceBrowser, ServicePublisher
from generic import ServiceResolvedInfo, ServiceInfo
from lib import pybonjour
import select
import sys,socket
import logging
import glib
import re

def parse_name(serviceName):
    res = re.match(
            "(?P<name>.*?)\.(?P<protocol>.*?)\.(?P<domain>.*?)\.",
            serviceName)
    if res:
        return res.group("name")
    else:
        print serviceName,"not matched!"

class BonjourServicePublisher(ServicePublisher):
    def __init__(self, name, regtype, port):
        '''
        Publish a service using the pybonjour backend, it works also on windows
        '''
        sdRef = pybonjour.DNSServiceRegister(name = name,
                                             regtype = regtype,
                                             port = port,
                                             callBack = self._register_callback)
        
        # Adding the processing of this stuff in the gobject mainloop.
        glib.io_add_watch(sdRef, glib.IO_IN, self._process_result )
    
    def _process_result(self, source, condition):
        '''
        Called by the glib mainloop when there's some service to process
        '''
        pybonjour.DNSServiceProcessResult(source)
        return True
    
    def _register_callback(self, sdRef, flags, errorCode, name, regtype, domain):
        if errorCode == pybonjour.kDNSServiceErr_NoError:
            self.on_connect()
        else:
            self.on_error("TODO: adding a good error message")
        
    def on_connect(self):
        ''
        
    def on_error(self,err):
        ''

class BonjourServiceBrowser(ServiceBrowser):
    '''
    Browse services using a bonjour compliant service.
    '''
    def __init__(self, type):
        browse_sdRef = pybonjour.DNSServiceBrowse(regtype = type,
                                                  callBack = self.browse_callback)
        self.process(browse_sdRef)
    
    def process(self, source, keep_cond=None):
        def callback(source, condition):
            pybonjour.DNSServiceProcessResult(source)
            if keep_cond:
                return keep_cond()
            else:
                return True
        
        return glib.io_add_watch(source, glib.IO_IN, callback)

    def browse_callback(self, sdRef, flags,
                        interfaceIndex, errorCode, serviceName,
                        regtype, replyDomain):
        logging.debug("Browsing")
        if errorCode != pybonjour.kDNSServiceErr_NoError:
            return self.on_error("TODO, adding an error message")
    
        if not (flags & pybonjour.kDNSServiceFlagsAdd):
            name = parse_name(serviceName)
            servinfo = ServiceInfo(
                                   interface=interfaceIndex, protocol=None,
                                   name=serviceName, type= regtype, domain=replyDomain)
            return self.on_service_removed(servinfo)
    
        logging.debug('Service added; resolving')
        resolve_sdRef = pybonjour.DNSServiceResolve(0,
                                                    interfaceIndex,
                                                    serviceName,
                                                    regtype,
                                                    replyDomain,
                                                    self.resolve_callback)
        self.process(resolve_sdRef)

    def resolve_callback(self, sdRef, flags, interfaceIndex,
                         errorCode, fullname,
                         hosttarget, port, txtRecord):
        if errorCode == pybonjour.kDNSServiceErr_NoError:
            servinfo = ServiceResolvedInfo(interface = interfaceIndex,
                                            protocol = None,
                                            name = parse_name(fullname),
                                            type = None,
                                            domain = None,
                                            host=hosttarget,
                                            address = socket.gethostbyname(hosttarget),
                                            port=port,
                                            txt=txtRecord)
            
            self.on_service_found(servinfo)

    def query_service(self, interface, fullname, rrtype, timeout=5):
        
        queried = [None]
        def callback(sdRef, flags, interface, error_code, fullname, rrtype,
                     rrclass, rdata, ttl):
            print "callbacking"
            if error_code == pybonjour.kDNSServiceErr_NoError:
                ip = socket.inet_ntoa(rdata)
                queried[0] = ip
                print "ip!!! ", ip
            else:
                print "Error! code: ",error_code
                queried[0] = False
        
        sdRef = pybonjour.DNSServiceQueryRecord(interfaceIndex=interface,
                                                fullname=fullname,
                                                rrtype=rrtype,
                                                callBack=callback)
        
        print "resolving ip"
        self.process(sdRef)

    def on_service_found(self, servinfo):
        print servinfo
        return
    def on_service_removed(self, servinfo):
        print servinfo
        return
    def on_error(self, err):
        return
def test_browse():
    BonjourServiceBrowser("_filesnake._tcp")
    glib.MainLoop().run()
def test_publish():
    BonjourServicePublisher("test","_filesnake._tcp",67332)
if __name__=="__main__":
    test_publish()
    test_browse()
