'''
This module defines data structures and interface that are used by the
various zeroconf backends.
'''
from abc import ABCMeta, abstractmethod
from collections import namedtuple

'''
Data structures that contains info about a connection discovered and
resolved.

before writing documnetation I have to sanitize these two in one data structure.
'''
#XXX they should be merged in only one, structure
ServiceResolvedInfo = namedtuple("ServiceResolvedInfo",
                                 'interface protocol name type domain host address port txt')

ServiceInfo = namedtuple("ServiceInfo",
                         "interface protocol name type domain")
    

'''
Interfaces for zeroconf related objects
'''
class ServiceBrowser(object):
    '''
    This class is used to discover services in the local network,
    it's abstract so you have to use one of its implementation.
    '''

    __metaclass__ = ABCMeta
    
    @abstractmethod
    def on_service_found(self, serviceinfo):
        '''
        This function is called when a new service is found
        '''
        return
    
    @abstractmethod
    def on_service_removed(self, serviceinfo):
        '''
        This method is called when a service has disappeared.
        '''
        return
    
    @abstractmethod
    def on_error(self, err):
        '''
        When something goes wrong this method is called, err is a
        message representing the reason of the error.
        '''
        return

class ServicePublisher(object):
    '''
    This class is used to publish a service in the zeroconf network
    (using a different server or using a zeroconf system service)
    '''
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def on_connect(self,):
        '''
        callback, called when the connection is estabilished with the
        spawned server or with the system service. (this can be
        intended also as on_success)
        '''
        return

    @abstractmethod
    def on_error(self,err):
        '''
        When something goes wrong this function is called
        '''
        return
